# general driver elements

ELEMENTS = [
    ### login
    {
        "name": "login",
        "classes": [],
        "text": [],
        "id": []
    },
    # username
    {
        "name": "loginUsername",
        "classes": [],
        "text": [],
        "id": []
    },
    # password
    {
        "name": "loginPassword",
        "classes": [],
        "text": [],
        "id": []
    },
    {
        "name": "loginCheck",
        "classes": ["b-make-post__streaming-link"],
        "text": [],
        "id": []
    },
    ### upload
    # send
    {
        "name": "new_post",
        "classes": ["g-btn.m-rounded", "button.g-btn.m-rounded"],
        "text": ["Post"],
        "id": []
    },


    # record voice
    {
        "name": "recordVoice",
        "classes": [None],
        "text": [],
        "id": []
    },
    # post price
    {
        "name": "post_price",
        "classes": [None],
        "text": [],
        "id": []
    },
    # post price cancel
    {
        "name": "post_price_cancel",
        "classes": [None],
        "text": [],
        "id": []
    },
    # post price save
    {
        "name": "post_price_save",
        "classes": [None],
        "text": [],
        "id": []
    },
    # go live
    {
        "name": "go_live",
        "classes": [None],
        "text": [],
        "id": []
    },
    # upload image file
    {
        "name": "image_upload",
        "classes": ["button.g-btn.m-rounded.b-chat__btn-submit", "g-btn.m-rounded.b-chat__btn-submit"],
        "text": [],
        "id": ["fileupload_photo"]
    },
    # show more options # unnecessary w/ tabbing
    {
        "name": "moreOptions",
        "classes": ["g-btn.m-flat.b-make-post__more-btn.has-tooltip", "g-btn.m-flat.b-make-post__more-btn", "button.g-btn.m-flat.b-make-post__more-btn"],
        "text": [],
        "id": []
    },
    
    # poll
    {
        "name": "poll",
        "classes": ["g-btn.m-flat.b-make-post__voting-btn", "g-btn.m-flat.b-make-post__voting-btn.has-tooltip", "button.g-btn.m-flat.b-make-post__voting-btn", "button.g-btn.m-flat.b-make-post__voting-btn.has-tooltip"],
        "text": ["<svg class=\"g-icon\" aria-hidden=\"true\"><use xlink:href=\"#icon-more\" href=\"#icon-more\"></use></svg>"],
        "id": []
    },
    # expire add
    {
        "name": "expiresAdd",
        "classes": ["b-make-post__expire-period-btn"],
        "text": ["Save"],
        "id": []
    },
    {
        "name": "expiresPeriods",
        "classes": ["b-make-post__expire__label"],
        "text": [],
        "id": []
    },
    {
        "name": "expiresSave",
        "classes": ["g-btn.m-rounded"],
        "text": ["Save"],
        "id": []
    },
    {
        "name": "expiresCancel",
        "classes": ["g-btn.m-rounded.m-border"],
        "text": ["Cancel"],
        "id": []
    },
    # poll cancel
    {
        "name": "pollCancel",
        "classes": ["b-dropzone__preview__delete"],
        "text": ["Cancel"],
        "id": []
    },
    # poll duration
    {
        "name": "pollDuration",
        "classes": ["g-btn.m-flat.b-make-post__voting__duration", "button.g-btn.m-flat.b-make-post__voting__duration", "g-btn.m-rounded.js-make-post-poll-duration-save", "button.g-btn.m-rounded.js-make-post-poll-duration-save"],
        "text": [],
        "id": []
    },
    # duration tabs
    {
        "name": "pollDurations",
        "classes": ["b-make-post__expire__label"],
        "text": [],
        "id": []
    },
    # poll save duration
    {
        "name": "pollSave",
        "classes": ["g-btn.m-rounded", "button.g-btn.m-rounded"],
        "text": ["Save"],
        "id": []
    },
    # poll add question
    {
        "name": "pollQuestionAdd",
        "classes": ["g-btn.m-flat.new_vote_add_option", "button.g-btn.m-flat.new_vote_add_option"],
        "text": [],
        "id": []
    },

    # expiration
    {
        "name": "expirationAdd",
        "classes": ["g-btn.m-flat.b-make-post__expire-period-btn", "button.g-btn.m-flat.b-make-post__expire-period-btn"],
        "text": [],
        "id": []
    },
    # expiration periods (same for duration)
    {
        "name": "expirationPeriods",
        "classes": ["b-make-post__expire__label", "button.b-make-post__expire__label"],
        "text": [],
        "id": []
    },
    # expiration save
    {
        "name": "expirationSave",
        "classes": ["g-btn.m-rounded", "button.g-btn.m-rounded", "button.g-btn.m-rounded.js-make-post-poll-duration-save", "g-btn.m-rounded.js-make-post-poll-duration-save"],
        "text": ["Save"],
        "id": []
    },
    # expiration cancel
    {
        "name": "expirationCancel",
        "classes": ["g-btn.m-rounded.m-border", "button.g-btn.m-rounded.m-border"],
        "text": ["Cancel"],
        "id": []
    },
    # discount modal for user
    {
        "name": "discountUserButton",
        "classes": ["g-btn.m-rounded"],
        "text": ["Apply"],
        "id": []
    },
    # discount save for user
    {
        "name": "discountUsers",
        "classes": ["b-users__item.m-fans"],
        "text": ["Save"],
        "id": []
    },

    ## price
    # price add
    {
        "name": "priceClick",
        "classes": ["g-btn.m-rounded"], # "b-chat__btn-set-price", "button.g-btn.m-rounded"
        "text": ["Save"],
        "id": []
    },
    # price enter (adds .00)
    {
        "name": "priceEnter",
        "classes": ["form-control.g-input", ".form-control.g-input", "input.form-control.g-input", "input.form-control.g-input"],
        "text": ["Free"],
        "id": []
    },

    # schedule add
    {
        "name": "scheduleAdd",
        "classes": ["g-btn.m-flat.b-make-post__datepicker-btn", "button.g-btn.m-flat.b-make-post__datepicker-btn"],
        "text": [],
        "id": []
    },
    # schedule next month
    {
        "name": "scheduleNextMonth",
        "classes": ["vdatetime-calendar__navigation--next", "button.vdatetime-calendar__navigation--next"],
        "text": [],
        "id": []
    },
    # schedule date
    {
        "name": "scheduleDate",
        "classes": ["vdatetime-calendar__current--month", "div.vdatetime-calendar__navigation > div.vdatetime-calendar__current--month", ".vdatetime-calendar__current--month", "div.vdatetime-calendar__current--month", "vdatetime-popup__date", "div.vdatetime-popup__date"],
        "text": [],
        "id": []
    },
    # schedule minutes
    {
        "name": "scheduleMinutes",
        "classes": ["vdatetime-time-picker__item", "button.vdatetime-time-picker__item"],
        "text": [],
        "id": []
    },
    # schedule hours
    {
        "name": "scheduleHours",
        "classes": ["vdatetime-time-picker__item.vdatetime-time-picker__item", "button.vdatetime-time-picker__item.vdatetime-time-picker__item"],
        "text": [],
        "id": []
    },
    # schedule days
    {
        "name": "scheduleDays",
        "classes": ["vdatetime-calendar__month__day", "button.vdatetime-calendar__month__day"],
        "text": [],
        "id": []
    },
    # schedule save
    {
        "name": "scheduleSave",
        "classes": ["g-btn.m-rounded", "button.g-btn.m-rounded"],
        "text": ["Save"],
        "id": []
    },

    ### message
    # message enter text
    {
        "name": "messageText",
        "classes": [".form-control.b-chat__message-input"],
        "text": [],
        "id": []
    },
    # message upload image
    {
        "name": "uploadImageMessage",
        "classes": ["g-btn.m-rounded.b-chat__btn-submit"],
        "text": [],
        "id": ["fileupload_photo"]
    },
    # upload error window close
    # tab probably closes error windows...
    {
        "name": "errorUpload",
        "classes": ["g-btn.m-rounded.m-border", "button.g-btn.m-rounded.m-border"],
        "text": ["Close"],
        "id": []
    },
    # messages all
    {
        "name": "messagesAll",
        "classes": ["b-chat__message__text"],
        "text": [],
        "id": []
    },
    # messages from user
    {
        "name": "messagesFrom",
        "classes": ["m-from-me"],
        "text": [],
        "id": []
    },

    ## Users
    {
        "name": "usersUsernames",
        "classes": ["g-user-username"],
        "text": [],
        "id": []
    },
    # users
    {
        "name": "usersUsers",
        "classes": ["g-user-name__wrapper", "b-username"],
        "text": [],
        "id": ["profileUrl"]
    },
    # users started dates
    {
        "name": "usersStarteds",
        "classes": ["b-fans__item__list__item"],
        "text": [],
        "id": []
    },
    # users ids
    {
        "name": "usersIds",
        "classes": ["a.g-btn.m-rounded.m-border.m-sm", "a.g-button.m-rounded.m-border.m-profile.m-with-icon.m-message-btn"],
        "text": [],
        "id": []
    },
    # users count
    {
        "name": "usersCount",
        "classes": ["l-sidebar__user-data__item__count", "b-tabs__nav__item.m-current"],
        "text": [],
        "id": []
    },
    {
        "name": "followingCount",
        "classes": ["b-tabs__nav__item.m-current"],
        "text": [],
        "id": []
    },
    # users discount buttons
    {
        "name": "discountUserButtons",
        "classes": ["g-btn.m-rounded.m-border.m-sm"],
        "text": [],
        "id": []
    },
    {
        "name": "newMessage",
        "classes": ["g-page__header__btn.b-chats__btn-new.has-tooltip"],
        "text": [],
        "href": ["/my/chats/send"],
        "id": [],
    },
    {
        "name": "messageAll",
        "classes": ["g-btn__text"],
        "text": ["Fans"],
        "id": [],
    },
    {
        "name": "messageRecent",
        "classes": ["g-btn__text"],
        "text": ["Recent"],
        "id": [],
    },
    {
        "name": "messageFavorite",
        "classes": ["g-btn__text"],
        "text": ["FAVORITE"],
        "id": [],
    },
    {
        "name": "messageRenewers",
        "classes": ["g-btn__text"],
        "text": ["Renew"],
        "id": [],
    },
    {
        "name": "promotionalTrial",
        "classes": ["g.btn.m-rounded.m-sm"],
        "text": ["Create new trial link"],
        "id": [],
    },
    {
        "name": "promotionalTrialCount",
        "classes": [],
        "text": [],
        "id": ["trial-count-select"],
    },
    {
        "name": "promotionalTrialExpiration",
        "classes": [],
        "text": [],
        "id": ["trial-expiration-select"],
    },
    {
        "name": "promotionalTrialDuration",
        "classes": [],
        "text": [],
        "id": ["promo-campaign-period-select"],
    },
    {
        "name": "promotionalTrialConfirm",
        "classes": ["g-btn.m-rounded"],
        "text": ["Create"],
        "id": [],
    },
    {
        "name": "promotionalTrialCancel",
        "classes": ["g-btn.m-rounded.m-border"],
        "text": ["Cancel"],
        "id": [],
    },
    {
        "name": "promotionalTrialLink",
        "classes": ["g.btn.m-rounded"],
        "text": ["Copy trial link"],
        "id": [],
    },
    {
        "name": "userOptions",
        "classes": ["btn.dropdown-toggle.btn-link"],
        "text": [],
        "id": ["__BVID__56__BV_toggle_"],
    },
    {
        "name": "discountUser",
        "classes": ["button"],
        "text": ["Give user a discount"],
        "id": [],
    },

    {
        "name": "promotionalTrialExpirationUser",
        "classes": [],
        "text": [],
        "id": ["trial-expire-select"],
    },
    {
        "name": "promotionalTrialDurationUser",
        "classes": [],
        "text": [],
        "id": ["trial-period-select"],
    },
    {
        "name": "promotionalTrialMessageUser",
        "classes": ["form-control.g-input"],
        "text": [],
        "id": [],
    },
    {
        "name": "promotionalTrialApply",
        "classes": ["g-btn.m-rounded"],
        "text": ["Apply"],
        "id": [],
    },
    {
        "name": "promotionalTrialCancel",
        "classes": ["g-btn.m-rounded.m-border"],
        "text": ["Cancel"],
        "id": [],
    },
    {
        "name": "numberOfPosts",
        "classes": ["b-profile__actions__count"],
        "text": [],
        "id": [],
    }


]


from datetime import datetime
from .classes import Poll, Schedule
from .driver import Driver
from .file import File, Google_File, Google_Folder
from .settings import Settings
from .user import User
from .validators import NumberValidator, TimeValidator, DateValidator, DurationValidator, ExpirationValidator, ListValidator
import PyInquirer
from PyInquirer import Validator, ValidationError

class Message():
	def __init__(self):
		self.text = None
		self.files = []
		##
		self.keywords = []
		self.tags = []
		self.performers = []
		self.hasPerformers = False
		## messages
		self.price = None
		self.recipients = [] # users to send to
		self.users = [] # prepared recipients
		## posts
		self.expiration = None
		self.poll = None
		self.schedule = None
		##
		self.gotten = False

	###########################################################################

	def backup_files(self):
		for file in self.files:
			file.backup()

	def delete_files(self):
		for file in self.files:
			file.delete()

	def cleanup_files(self):
		self.backup_files()
		self.delete_files()

	@staticmethod
	def format_keywords(keywords):
		if len(keywords) > 0: return " #{}".format(" #".join(keywords))
		return ""

	@staticmethod
	def format_performers(performers): # spaced added after @ to close performer search modal
		if len(performers) > 0: return " w/ @{} ".format(" @".join(performers))
		return ""
			
	@staticmethod
	def format_tags(tags):
		if len(tags) > 0: return " @{}".format(" @".join(tags))
		return ""

	def format_text(self):
		return "{}{}{}{}".format(self.get_text(), Message.format_performers(self.get_performers()), Message.format_tags(self.get_tags()),
			Message.format_keywords(self.get_keywords())).strip()

	def get_keywords(self):
		if str(self.keywords) == "unset": return []
		# if self.keywords: return self.keywords
		if len(self.keywords) > 0: return self.keywords
		keywords = Settings.get_keywords() or []
		if len(keywords) > 0: return keywords
		if not Settings.prompt("keywords"):
			self.keywords = "unset"
			return []
		question = {
			'type': 'input',
			'name': 'keywords',
			'message': 'Keywords:',
			'validate': ListValidator
		}
		answers = PyInquirer.prompt(question)
		keywords = answers["keywords"]
		keywords = keywords.split(",")
		keywords = [n.strip() for n in keywords]
		if not Settings.confirm(keywords): return self.get_keywords()
		self.keywords = keywords
		return self.keywords

	def get_performers(self):
		if str(self.performers) == "unset": return []
		# if self.performers: return self.performers
		if len(self.performers) > 0: return self.performers
		performers = Settings.get_performers() or []
		if len(performers) > 0: return performers

		if len(self.files) > 0:
			for file in self.files:
				if hasattr(file, "performer"):
					p = getattr(file, "performer")
					if p not in performers:
						performers.append(p)
			if len(performers) > 0:
				self.performers = performers
				return performers

		if not Settings.prompt("performers"):
			self.performers = "unset"
			return []
		question = {
			'type': 'input',
			'name': 'performers',
			'message': 'Performers:',
			'validate': ListValidator
		}
		answers = PyInquirer.prompt(question)
		performers = answers["performers"]
		performers = performers.split(",")
		performers = [n.strip() for n in performers]
		if not Settings.confirm(performers): return self.get_performers()
		self.performers = performers
		return self.performers

	def get_tags(self):
		if str(self.tags) == "unset": return []
		# if self.tags: return self.tags
		if len(self.tags) > 0: return self.tags
		tags = Settings.get_tags() or []
		if len(tags) > 0: return tags
		if not Settings.prompt("tags"):
			self.tags = "unset"
			return []
		question = {
			'type': 'input',
			'name': 'tags',
			'message': 'Tags:',
			'validate': ListValidator
		}
		answers = PyInquirer.prompt(question)
		tags = answers["tags"]
		tags = tags.split(",")
		tags = [n.strip() for n in tags]
		if not Settings.confirm(tags): return self.get_tags()
		self.tags = tags
		return self.tags

	# ensures File references exist and are downloaded
	# files are File references
	# file references can be GoogleId references which need to download their source
	# files exist when checked for size
	# ?
	def get_files(self):
		if str(self.files) == "unset": return []
		if len(self.files) > 0: return self.files[:int(Settings.get_upload_max())]
		if (Settings.is_prompt() and not Settings.prompt("upload files")) or (not Settings.is_prompt() and Settings.get_category() == None):
			self.files = "unset"
			return []
		files = []
		if len(self.files) == 0 and len(Settings.get_input()) > 0:
			files.append(Settings.get_input_as_files())
		elif len(self.files) == 0:
			files = File.select_file_upload_method()
		if files == None: files = []
		if Settings.get_source() == "google" and len(files) == 0 and len(Google_File.get_files()) > 0:
			files = Google_File.select_files()
		# elif Settings.get_source() == "dropbox" and len(files) == 0 and len(Dropbox.get_files()) > 0:
		# 	files = Dropbox.select_files()
		elif Settings.get_source() == "remote" and len(files) == 0 and len(Remote.get_files()) > 0:
			files = Remote.select_files()
		elif Settings.get_source() == "local" and len(files) == 0 and len(File.get_files()) > 0:
			files = File.select_files()
		filed = []
		for file in files:
			if isinstance(file, Google_Folder): filed.extend(file.get_files())
			else:
				if hasattr(file, "performer"):
					self.hasPerformers = True
				filed.append(file)
		self.files = filed[:int(Settings.get_upload_max())]
		return self.files

	def get_expiration(self):
		if str(self.expiration) == "unset": return None
		if self.expiration: return self.expiration
		expires = Settings.get_expiration() or None
		if expires: return expires
		if not Settings.prompt("expiration"):
			self.expiration = "unset"
			return None
		question = {
			'type': 'input',
			'name': 'expiration',
			'message': 'Expiration [1, 3, 7, 99 (\'No Limit\')]',
			'validate': ExpirationValidator
		}
		answers = PyInquirer.prompt(question)
		expiration = answers["expiration"]
		if not Settings.confirm(expiration): return self.get_expiration()
		self.expiration = expiration
		return self.expiration

	def get_poll(self):
		if str(self.poll) == "unset": return None
		if self.poll and self.poll.check(): return self.poll
		if not Settings.prompt("poll"):
			self.poll = "unset"
			return None
		poll = Poll()
		poll.get()
		if not poll.check(): return None
		self.poll = poll
		return poll

	def get_price(self):
		if self.price: return self.price
		price = Settings.get_price() or None
		if price: return price
		if not Settings.prompt("price"): return ""
		question = {
			'type': 'input',
			'name': 'price',
			'message': 'Price',
			'validate': NumberValidator,
			'filter': lambda val: int(val)
		}
		answers = PyInquirer.prompt(question)
		price = answers["price"]
		if not Settings.confirm(price): return self.get_price()
		self.price = price
		return self.price

	# ensures listed recipients are users
	# Settings.USERS and self.recipients should be usernames
	# if includes [all, recent, favorite] & usernames it only uses the 1st found of [all,...]
	def get_recipients(self):
		if len(self.users) > 0: return self.users
		users = []
		if len(self.recipients) == 0 and len(Settings.get_users()) > 0: 
			users = Settings.get_users()
		elif len(self.recipients) == 0 and Settings.get_user(): 
			users = [Settings.get_user()]
		elif len(self.recipients) == 0:
			users = User.select_users()
		# users = []
		# for user in recipients:
		#     if str(user.username).lower() == "all":
		#         users = User.get_all_users()
		#         break
		#     elif str(user.username).lower() == "recent":
		#         users = User.get_recent_users()
		#         break
		#     elif str(user.username).lower() == "favorite":
		#         users = User.get_favorite_users()
		#         break
		#     else: users.append(user)
		self.users = users
		return self.users

	def get_schedule(self):
		if str(self.schedule) == "unset": return None
		if self.schedule: return self.schedule
		if not Settings.prompt("schedule"):
			self.schedule = "unset"
			return None
		schedule = Schedule()
		schedule.get()
		if not schedule.check(): return None
		self.schedule = schedule
		return schedule
		
	def get_text(self):
		if self.text: return self.text
		text = Settings.get_text() or None
		if text: 
			self.text = text
			return text
		if not Settings.prompt("text"): return None
		question = {
			'type': 'input',
			'name': 'text',
			'message': 'Text:'
		}
		answers = PyInquirer.prompt(question)
		text = answers["text"]
		if not Settings.confirm(text): return self.get_text()
		self.text = text
		return self.text

	def get(self):
		if self.gotten: return
		self.get_text()
		self.get_keywords()
		self.get_tags()
		self.get_price()
		self.get_poll()
		self.get_schedule()
		self.get_expiration()
		self.get_files()
		self.get_recipients()
		if not self.text:
			if len(self.files) > 0:
				self.text = self.files[0].get_title()
		if Settings.get_performer_category() or self.hasPerformers:
			self.get_performers()
		else: # might as well skip asking if not pulling from performer category
			self.performers = "unset"
		self.gotten = True

	def get_post(self):
		if self.gotten: return
		self.get_text()
		self.get_keywords()
		self.get_tags()
		self.get_poll()
		self.get_schedule()
		self.get_expiration()
		self.get_files()
		if not self.text:
			if len(self.files) > 0:
				self.text = self.files[0].get_title()
		if Settings.get_performer_category() or self.hasPerformers:
			self.get_performers()
		else:
			self.performers = "unset"
		self.gotten = True

	def get_message(self):
		if self.gotten: return
		self.get_recipients()
		self.get_text()
		self.get_price()
		self.get_files()
		if not self.text:
			if len(self.files) > 0:
				self.text = self.files[0].get_title()
		if Settings.get_performer_category() or self.hasPerformers:
			self.get_performers()
		else:
			self.performers = "unset"
		self.gotten = True

	@staticmethod
	def Post():
		message = Message()
		message.post()

	def post(self):
		self.get_post()
		if Settings.is_prompt():
			if not Settings.prompt("Post"): return
		successful = False
		try: successful = Driver.post(self)
		except Exception as e:
			Settings.dev_print(e)
			successful = False
		if successful: self.cleanup_files()

		# this should all be here instead of Driver.post
		# print("Posting:")
  #       print("- Files: {}".format(len(files)))
  #       print("- Keywords: {}".format(keywords))
  #       print("- Performers: {}".format(performers))
  #       print("- Tags: {}".format(tags))
  #       print("- Text: {}".format(text))
  #       print("- Tweeting: {}".format(Settings.is_tweeting()))


	@staticmethod
	def Send():
		message = Message()
		message.send()

	# sends to recipients
	# 'post' as recipient will post message instead
	def send(self):
		self.get_message()
		if Settings.is_prompt():
			if not Settings.prompt("Send"): return
		successful = False
		try: 
			# for user in self.get_recipients():
			for user in self.users:
				# if isinstance(user, str) and str(user) == "post": successful_ = Driver.post(self)
				# print("Messaging: {}".format(user.username))
				if isinstance(user, User): successful_ = User.message_user(user.username, self)
				else: successful_ = User.message_user(user, self)
				if not successful_: continue
				successful_ = Driver.message(user.username)
		except Exception as e:
			Settings.dev_print(e)
			successful = False
		if successful: self.cleanup_files()


#!/usr/bin/python3
# main OnlySnarf class

import random
import os
import shutil
import datetime
import json
import sys
import pathlib
import time
##
from .settings import Settings

#####################
##### OnlySnarf #####
#####################

class Snarf:

    def __init__(self):
        pass

    ####################
    ##### Discount #####
    ####################

    @staticmethod
    def discount(discount=None):
        from .classes import Discount
        if not discount: discount = Discount()
        try: discount.apply()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()

    ################
    ##### Exit #####
    ################

    @staticmethod
    def exit():
        exit()

    #################
    ##### Login #####
    #################

    @staticmethod
    def login():
        from .driver import Driver
        loggedIn = False
        try: loggedIn = Driver.auth()
        except Exception as e: Settings.dev_print(e)
        return loggedIn

    ###################
    ##### Message #####
    ###################

    @staticmethod
    def message(message=None):
        from .message import Message
        if not message: message = Message()
        try: message.send()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()
                
    ################
    ##### Post #####
    ################

    @staticmethod
    def post(message=None):
        from .message import Message
        if not message: message = Message()
        try: message.post()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()

    ###################
    ##### Profile #####
    ###################

    @staticmethod
    def profile(profile=None):
        from .profile import Profile
        if not profile: profile = Profile()
        try: 
            # get profile method
            method = Settings.get_profile_method()
            if method == "backup":
                profile.backup_content()
            elif method == "syncfrom":
                Profile.sync_from_profile()
            elif method == "syncto":
                Profile.sync_to_profile()
            else: print("{}: Missing Profile Method".format(colorize("Error","red")))
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()
        
    #####################
    ##### Promotion #####
    #####################

    @staticmethod
    def promotion(promotion=None):
        from .classes import Promotion
        if not promotion: promotion = Promotion()
        try: 
            # get promotion method
            method = Settings.get_promotion_method()
            if method == "user":
                promotion.apply_to_user()
            elif method == "trial":
                promotion.create_trial_link()
            else: print("{}: Missing Promotion Method".format(colorize("Error","red")))
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()

    #################
    ##### Reset #####
    #################

    @staticmethod
    def reset():
        from .driver import Driver
        Driver.reset()

    #################
    ##### Users #####
    #################

    @staticmethod
    def get_following():
        from .user import User
        users = []
        try: users = User.get_following()
        except Exception as e: Settings.dev_print(e)
        return users

    @staticmethod
    def get_users():
        from .user import User
        users = []
        try: users = User.get_all_users()
        except Exception as e: Settings.dev_print(e)
        return users

    ###############
    ##### Dev #####
    ###############

    @staticmethod
    def test():
        from .driver import Driver
        from .user import User
        # from . import cron as Cron
        # print('0/3 : Deleting Locals')
        print('1/3 : Testing')
        # print('TESTING: Users')
        # response = Driver.users_get()
        # return True
        print('TESTING: Following')
        response = User.get_following()
        # return True
        # print('TESTING: Settings - Get')
        # response = Driver.settings_get_all()
        # return True
        # print('TESTING: Cron')
        # response = Cron.test()
        # if not response or response == None:
        #     print("Error: Failed to test crons")
        # reset_ = reset()
        # if not reset_:
        #     return print("Error: Failed to Reset")
        return True

################################################################################################################################################

import atexit
def exit_handler():
    print("Shnarrf?")
    exit()
atexit.register(exit_handler)

# import signal
# def signal_handler(sig, frame):
#     print('Shnnnarf?')
#     exit()
# signal.signal(signal.SIGINT, signal_handler)
  
def exit():
    from OnlySnarf.src.driver import Driver
    Driver.exit()
    sys.exit(0)

def main():
    try:
        from .file import File
        # File.remove_local()
        Settings.set_prompt(False)
        Settings.set_confirm(False)
        action = Settings.get_action()
        print("Running - {}".format(action))
        ## Actions
        success = False
        if str(action) == "test":
            success = Snarf.test()
        elif str(action) == "post":
            success = Snarf.post()
        elif str(action) == "message":
            success = Snarf.message()
        elif str(action) == "discount":
            success = Snarf.discount()
        elif str(action) == "promotion":
            success = Snarf.promotion()
        elif str(action) == "profile":
            success = Snarf.profile()
        else:
            print("Warning: Missing Method")
        Snarf.exit()
    except Exception as e:
        Settings.dev_print(e)
        print("Shnarf!")
    finally:
        sys.exit(0)

################################################################################################################################################

if __name__ == "__main__":
    main()


import os, shutil, random, sys
from .ffmpeg import ffmpeg
from . import google as Google
from .settings import Settings
import PyInquirer
from . import remote as Remote
from PIL import Image
from os import walk

ONE_GIGABYTE = 1000000000
ONE_MEGABYTE = 1000000
FIFTY_MEGABYTES = 50000000
ONE_HUNDRED_KILOBYTES = 100000

MIMETYPES_IMAGES = "(mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"
MIMETYPES_VIDEOS = "(mimeType contains 'video/mp4' or mimeType contains 'video/quicktime' or mimeType contains 'video/x-ms-wmv' or mimeType contains 'video/x-flv')"
MIMETYPES_ALL = "(mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png' or mimeType contains 'video/mp4' or mimeType contains 'video/quicktime')"

MIMETYPES_IMAGES_LIST = ["image/jpeg","image/jpg","image/png"]
MIMETYPES_VIDEOS_LIST = ["video/mp4","video/quicktime","video/x-ms-wmv","video/x-flv"]
MIMETYPES_ALL_LIST = []
MIMETYPES_ALL_LIST.extend(MIMETYPES_IMAGES_LIST)
MIMETYPES_ALL_LIST.extend(MIMETYPES_VIDEOS_LIST)

def print_same_line(text):
    sys.stdout.write('\r')
    sys.stdout.flush()
    sys.stdout.write(text)
    sys.stdout.flush()

###############################################################

class File():
    FILES = None

    def __init__(self):
        self.path = None
        self.ext = None
        self.type = None
        ##
        self.title = None
        self.category = None # [image, gallery, video, performer]
        ##
        self.remote_path = None

    ######################################################################################

    # move to backup folder in GDrive
    # Google.move_file
    # Google.move_files
    def backup(self):
        if not File.backup_text(self.get_title()): return
        if Settings.get_destination() == "remote":
            Remote.backup_file(self)
        elif Settings.get_destination() == "google":
            Google.upload_file(file=self)
        # elif Settings.get_destination() == "dropbox":
            # Dropbox.upload_file(file=self)
        else:
            # move file to local backup location
            backupPath = os.path.join(Settings.get_local_path(), "posted")
            backupPath = os.path.join(backupPath, file.category, file.title)
            shutil.move(file.get_path(), backupPath)

    @staticmethod
    def backup_text(title):
        if Settings.is_skip_download():
            Settings.maybe_print("Warning: Skipping Backup, skipped download")
            return False
        if Settings.is_force_backup():
            Settings.maybe_print("Backing Up (forced): {}".format(title))
        elif not Settings.is_backup():
            Settings.maybe_print('Skipping Backup (disabled): {}'.format(title))
            return False
        elif Settings.is_debug():
            Settings.maybe_print("Skipping Backup (debug): {}".format(title))
            return False
        else:
            Settings.maybe_print('Backing Up: {}'.format(title))
        return True

    @staticmethod
    def backup_files(files=[]):
        if files[0] and files[0].remote_path:
            Remote.backup_files(files)
        else:
            for file in files:
                file.backup()
        return True

    def check_size(self):
        if not os.path.exists(self.get_path()): return False
        size = os.path.getsize(self.get_path())
        Settings.maybe_print("File Size: {}kb - {}mb".format(size/1000, size/1000000))
        global ONE_MEGABYTE
        if size <= ONE_MEGABYTE:
            Settings.maybe_print("Warning: Small File Size")
        global ONE_HUNDRED_KILOBYTES
        if size <= ONE_HUNDRED_KILOBYTES:
            Settings.maybe_print("Warning: Tiny File Size")
        if size == 0:
            Settings.maybe_print("Error: Empty File Size")
            return False
        return True

    # def combine(self):
    #     if len(self.files) == 0: return
    #     Settings.dev_print("combining files: {}".format(len(self.files)))
    #     Settings.dev_print("combine path: {}".format(combinedPath))
    #     combinedPath = os.path.join(File.get_tmp(), "{}-combined".format(self.title))
    #     for file in files:
    #         shutil.move(file.get_path(), combinedPath)
    #         file.path = "{}/{}".format(combinedPath, self.title)
    #     self.combined = ffmpeg.combine(combinedPath)

    ##############################

    # Deletes online file
    def delete(self):
        if not File.delete_text(self.get_title()): return
        if self.remote_path:
            Remote.delete_file(self)
        else:
            try: 
                os.remove(self.get_path())
                print('File Deleted: {}'.format(self.get_title()))
            except Exception as e: Settings.dev_print(e)

    @staticmethod
    def delete_text(title):
        if Settings.is_skip_download():
            Settings.maybe_print("Warning: Skipping Delete, skipped download")
            return False
        if not Settings.is_delete():
            Settings.maybe_print('Skipping Delete (disabled): {}'.format(title))
            return False
        elif Settings.is_debug():
            Settings.maybe_print("Skipping Delete (debug): {}".format(title))
            return False
        else:
            Settings.maybe_print('Deleting: {}'.format(title))
        return True

    ##############################

    def get_files():
        pass
        # return all local files from Settings.mount_path or whatever

    def get_ext(self):
        if self.ext: return self.ext
        self.get_title()

    def get_path(self):
        if not self.path:
            Settings.maybe_print("Error: Missing File Path")
            return  ""
        return self.path

    def get_title(self):
        if str(self.title) != "": return self.title
        path = self.get_path()
        if str(path) == "": 
            Settings.maybe_print("Error: Missing File Title")
            return ""
        title, ext = os.path.splitext(path)
        self.ext = ext
        self.title = title
        return self.title

    # def get_type(self):
    #     if str(self.type) != "": return self.type
    #     if (self.get_ext()) in MIMETYPES_IMAGES_LIST:
    #         self.type = Image()
    #     elif (self.get_ext()) in MIMETYPES_VIDEOS_LIST:
    #         self.type = Video()
    #     setattr(self.type, "path", self.get_path())
    #     return self.type

    @staticmethod
    def get_tmp():
        tmp = os.getcwd()
        if Settings.get_mount_path() != "":
            tmp = os.path.join(Settings.get_mount_path(), "tmp")
        else:
            tmp = os.path.join(tmp, "tmp")
        if not os.path.exists(str(tmp)):
            os.mkdir(str(tmp))
        return tmp

    def get_type(self):
        if self.type: return self.type
        if str(self.get_ext()) in str(MIMETYPES_VIDEOS_LIST):
            self.type = Video()
        elif str(self.get_ext()) in str(MIMETYPES_IMAGES_LIST):
            self.type = Image()
        else: print("Warning: Unable to Parse File Type")
        return self.type

    # files are File references
    # file references can be GoogleId references which need to download their source
    # files exist when checked for size
    def prepare(self):
        self.get_type().prepare()
        if not self.check_size():
            return False
        return True

    # Deletes all local files
    @staticmethod
    def remove_local():
        try:
            # if str(Settings.SKIP_DELETE) == "True":
                # Settings.maybe_print("Skipping Local Remove")
                # return
            # print('Deleting Local File(s)')
            # delete /tmp
            tmp = File.get_tmp()
            if os.path.exists(tmp):
                shutil.rmtree(tmp)
                print('Local File(s) Removed')
            else:
                print('Local Files Not Found')
        except Exception as e:
            Settings.dev_print(e)

    @staticmethod
    def get_files():
        if File.FILES: return File.FILES
        category = Settings.get_category()
        if not category: category = Settings.select_category()
        if not category: Settings.dev_print("Warning: Missing Category")
        files = File.get_files_by_category(category)
        if Settings.get_title():
            for file in files:
                if str(Settings.get_title()) == str(file.get_title()):
                    files = [file]
                    break
        File.FILES = files
        return files

    @staticmethod
    def get_files_by_folder(path):
        f = []
        for (dirpath, dirnames, filenames) in walk(path):
            f.extend(filenames)
            break
        return f

    def get_folder_by_name(category, parent=None):
        if not parent:
            parent = Settings.get_local_path()
        f = []
        for (dirpath, dirnames, filenames) in walk(parent):
            for dir_ in dirnames:
                if str(dir_) == str(category):
                    return dir_
            break
        return None

    @staticmethod
    def get_folders_of_folder_by_keywords(categoryFolder):
        if categoryFolder == None: return []
        folders = File.get_folders_of_folder(categoryFolder)
        folders_ = []
        for folder in folders:
            if Settings.get_drive_keyword() and str(folder.get_title()) != str(Settings.get_drive_keyword()):
                Settings.dev_print('{} -> not keyword'.format(folder.get_title()))
                continue
            elif Settings.get_drive_ignore() and str(folder.get_title()) == str(Settings.get_drive_ignore()):
                Settings.dev_print('{} -> by not keyword'.format(folder.get_title()))
                continue
            elif str(folder.get_title()) == str(Settings.get_drive_keyword):
                Settings.dev_print('{} -> by keyword'.format(folder.get_title()))
            else:
                Settings.dev_print("{}".format(folder.get_title()))
            folders_.append(folder)
        return folders_

    @staticmethod
    def get_random_file():
        return random.choice(Google_File.get_files())

    @staticmethod
    def get_images_of_folder(folder):
        imgs = []
        files = []
        path = folder.get_path()
        valid_images = [".jpg",".gif",".png",".tga",".jpeg"]
        for f in os.listdir(path):
            ext = os.path.splitext(f)[1]
            if ext.lower() not in valid_images:
                continue
            file = File()
            setattr(file, "path", os.path.join(path,f))
            files.append(file)
        return files

    @staticmethod
    def get_videos_of_folder(folder):
        videos = []
        files = []
        path = folder.get_path()
        valid_videos = [".mp4",".mov"]
        for f in os.listdir(path):
            ext = os.path.splitext(f)[1]
            if ext.lower() not in valid_videos:
                continue
            file = File()
            setattr(file, "path", os.path.join(path,f))
            files.append(file)
        return files

    @staticmethod
    def get_folders_of_folder(folder):
        # os.walk(directory)
        # will yield a tuple for each subdirectory. Ths first entry in the 3-tuple is a directory name, so
        # [x[0] for x in os.walk(directory)]
        # should give you all of the subdirectories, recursively.
        # Note that the second entry in the tuple is the list of child directories of the entry in the first position, so you could use this instead, but it's not likely to save you much.
        # However, you could use it just to give you the immediate child directories:
        folders = []
        for folder in (os.walk(folder.get_path()))[1]:
            fol = Folder()
            setattr(fol, "path", folder)
            folders.append(fol)
        return folders

    @staticmethod
    def get_files_by_category(cat, performer=None):
        Settings.maybe_print("Loading Local Files...")
        files = []
        ##
        def parse_categories(category, categoryFolder=None):
            files = []
            # return File.get_files_by_category(cat)
            if "image" in str(category):
                categoryFolder = File.get_folder_by_name(category, parent=categoryFolder)
                for folder in File.get_folders_of_folder_by_keywords(categoryFolder):
                    for image in File.get_images_of_folder(folder):
                        file = File()
                        setattr(file, "path", image)
                        setattr(file, "category", folder)
                        files.append(file)
            elif "video" in str(category):
                categoryFolder = File.get_folder_by_name(category, parent=categoryFolder)
                for folder in File.get_folders_of_folder_by_keywords(categoryFolder):
                    videos = File.get_videos_of_folder(folder)
                    if len(videos) > 0:
                        files.append(folder.get_title())
                    for video in videos:
                        file = File()
                        setattr(file, "path", video)
                        setattr(file, "category", folder)
                        files.append(file)
            elif "galler" in str(category):
                categoryFolder = File.get_folder_by_name(category, parent=categoryFolder)
                for folder in File.get_folders_of_folder_by_keywords(categoryFolder):
                    galleries = File.get_folders_of_folder(folder)
                    if len(galleries) > 0:
                        files.append(folder.get_title())
                    for gallery in galleries:
                        file = Folder()
                        setattr(file, "path", gallery)
                        setattr(file, "category", folder)
                        files.append(file)
            elif "performer" in str(category):
                categoryFolder = File.get_folder_by_name(category, parent=categoryFolder)
                for performer in File.get_folders_of_folder_by_keywords(categoryFolder):
                    # for performer in File.get_folders_of_folder(folder):
                    p = Folder()
                    setattr(p, "path", performer)
                    setattr(p, "category", categoryFolder)
                    files.append(p)
            return files
        ##
        if performer:
            categoryFolder = File.get_folder_by_name("performers")
            for performerFolder in File.get_folders_of_folder_by_keywords(categoryFolder):
                if str(performer) == str(performerFolder.get_title()):
                    return parse_categories(cat, categoryFolder=performerFolder)
        return parse_categories(cat)

    @staticmethod
    def select_file(category, performer=None):
        files = File.get_files_by_category(category, performer=performer)
        files_ = []
        for file in files:
            if isinstance(file, str):
                files_.append(PyInquirer.Separator())
                continue
            file.category = category
            file_ = {
                "name": file.get_title(),
                "value": file,
            }
            files_.append(file_)
        if len(files_) == 0:
            print("Missing Files")
            return File.select_files()
        question = {
            'type': 'list',
            'name': 'file',
            'message': 'File Path:',
            'choices': files_,
            # 'filter': lambda file: file.lower()
        }
        answer = PyInquirer.prompt(question)
        file = answer["file"]
        if not Settings.confirm(file.get_path()): return None
        return file

    @staticmethod
    def select_files():
        if not Settings.is_prompt(): return [File.get_random_file()]
        category = Settings.select_category()
        if not category: return File.select_file_upload_method()
        # if not Settings.confirm(category): return File.select_files()
        print("Select Files or a Folder")
        files = []
        while True:
            file = File.select_file(category)
            if not file: break
            ##
            if "performer" in str(category):
                cat = Settings.select_category([cat for cat in Settings.get_categories() if "performer" not in cat])
                performerName = file.get_title()
                file = File.select_file(cat, performer=performerName)
                if not file: break
                setattr(file, "performer", performerName)
                files.append(file)
                if "galler" in str(cat) or "video" in str(cat): break
            ##
            files.append(file)
            if "galler" in str(category) or "video" in str(category): break
        if not Settings.confirm([file.get_title() for file in files]): return File.select_files()
        return files

    @staticmethod
    def select_file_upload_method():
        if str(Settings.get_source()) != "": return []
        if not Settings.prompt("upload files"): return []
        print("Select an upload source")
        sources = Settings.get_source_options()
        question = {
            'type': 'list',
            'name': 'upload',
            'message': 'Upload:',
            'choices': [src.title() for src in sources]
        }
        upload = PyInquirer.prompt(question)["upload"]
        # if not Settings.confirm(upload): return File.select_file_upload_method()
        if str(upload) == "Google":
            return Google_File.select_files()
        # elif str(upload) == "Dropbox":
            # return Dropbox.select_files()
        elif str(upload) == "Remote":
            return Remote.select_files()
        return File.select_files()

    def upload(self):
        if not self.prepare():
            print("Error: Unable to Upload File - {}".format(self.get_title()))
            return False
        self.backup()
        self.delete()
        return True


class Folder(File):
    def __init__(self):
        File.__init__(self)
        self.files = None

    def backup(self):
        if File.backup_text(self.get_title()): return
        # Google.upload_gallery(files=self.files)

    def check_size(self):
        for file in self.get_files():
            exists = file.check_size()
            if not exists: return False
        return True

    def download(self):
        print("Downloading Folder: {}".format(self.get_title()))
        if len(self.files) == 0:
            file_list = Google.get_files_by_folder_id(self.get_id())
            self.files = []
            for file in file_list:
                file_ = Google_File()
                setattr(file_, "id", file["id"])
                setattr(file_, "file", file)
                self.files.append(file_)
        folder_size = len(self.files)
        Settings.maybe_print('Folder size: {}'.format(folder_size))
        Settings.maybe_print('Upload limit: {}'.format(Settings.get_upload_max()))
        if int(folder_size) == 0:
            print('Error: Empty Folder')
            return False
        file_list = self.files
        random.shuffle(file_list)
        file_list = file_list[:int(Settings.get_upload_max())]
        ## video preference
        videos = []
        for file in file_list:
            if str(file.get_mimetype()) in MIMETYPES_VIDEOS_LIST:
                videos.append(file)
        if len(videos) > 0: file_list = [random.choice(videos)]
        ##
        i = 1
        for file in sorted(file_list, key = lambda x: x.get_title()):
            # print_same_line("Downloading: {} ({}/{})".format(file.get_title(), i, folder_size))
            print("Downloading: {} ({}/{})".format(file.get_title(), i, folder_size))
            file.download()
            i+=1
        print()
        print("Downloaded Folder: {}".format(self.get_title()))

    def get_files(self):
        if not self.files and self.path:
            self.files = []
            files = File.get_files_by_folder(self.get_path())
            for file in files:
                file_ = Google_File()
                setattr(file_, "file", file)
                self.files.append(file_)
        if Settings.get_title():
            for file in self.files:
                if str(Settings.get_title()) == str(file.get_title()):
                    self.files = [file]
                    break
        return self.files


###################################################################################

class Google_File(File):

    def __init__(self):
        File.__init__(self)
        self.id = None
        self.title = None
        self.file = None
        self.parent = None
        self.mimeType = None

    def backup(self):
        if not File.backup_text(self.get_title()): return
        Google.backup_file(self)

    def delete(self):
        if not File.delete_text(self.get_title()): return
        Google.delete(self)

    @staticmethod
    def download_text(title):
        if Settings.is_skip_download():
            print("Skipping Download (debug)")
            return False
        return True

    @staticmethod
    def download_files(files=[]):
        Settings.maybe_print('Download limit: '+str(Settings.get_image_download_limit()))
        random.shuffle(files)
        files = files[:int(Settings.get_image_download_limit())]
        print('Downloading Files: {}'.format(len(files)))
        i = 1
        for file in sorted(files, key = lambda x: x['title']):
            print('Downloading: {}/{}'.format(i, Settings.get_image_download_limit()))
            file.download()
        print("Downloaded: {}".format(len(files)))

    # Download File
    def download(self):
        if not Google_File.download_text(self.title): return False
        successful = Google.download_file(self)
        if not successful: return False
        ### Finish ###
        if not self.check_size():
            print("Error: Missing Downloaded File")
            return False
        print("Downloaded: {}".format(self.title))
        return True

    def get_ext(self):
        if self.ext: return self.ext
        title, ext = os.path.splitext(self.get_file()["title"])
        if str(ext) == "":
            ext = self.get_file()["mimeType"]
            mime, ext = str(ext).split("/")
            ext = "."+str(ext)
        self.ext = ext
        self.title = title
        return self.ext

    def get_id(self):
        if self.id: return self.id
        if self.file: self.id = self.file["id"]
        return self.id

    def get_file(self):
        if self.file: return self.file
        self.file = Google.get_file(self.get_id())
        # if not self.check_size(): self.download()
        return self.file

    @staticmethod
    def get_files():
        if File.FILES: return File.FILES
        category = Settings.get_category()
        if not category: category = Settings.select_category()
        if not category: Settings.dev_print("Warning: Missing Category")
        files = Google_File.get_files_by_category(category)
        if Settings.get_title():
            for file in files:
                if str(Settings.get_title()) == str(file.get_title()):
                    files = [file]
                    break
        File.FILES = files
        return files

    @staticmethod
    def get_files_by_category(cat, performer=None):
        Settings.maybe_print("Loading Google Files...")
        files = []
        ##
        def parse_categories(category, categoryFolder=None):
            files = []
            # return Google_File.get_files_by_category(cat)
            if "image" in str(category):
                categoryFolder = Google.get_folder_by_name(category, parent=categoryFolder)
                for folder in Google.get_folders_of_folder_by_keywords(categoryFolder):
                    for image in Google.get_images_of_folder(folder):
                        file = Google_File()
                        setattr(file, "file", image)
                        setattr(file, "parent", folder)
                        files.append(file)
            elif "video" in str(category):
                categoryFolder = Google.get_folder_by_name(category, parent=categoryFolder)
                for folder in Google.get_folders_of_folder_by_keywords(categoryFolder):
                    videos = Google.get_videos_of_folder(folder)
                    if len(videos) > 0:
                        files.append(folder["title"])
                    for video in videos:
                        file = Google_File()
                        setattr(file, "file", video)
                        setattr(file, "parent", folder)
                        files.append(file)
            elif "galler" in str(category):
                categoryFolder = Google.get_folder_by_name(category, parent=categoryFolder)
                for folder in Google.get_folders_of_folder_by_keywords(categoryFolder):
                    galleries = Google.get_folders_of_folder(folder)
                    if len(galleries) > 0:
                        files.append(folder["title"])
                    for gallery in galleries:
                        file = Google_Folder()
                        setattr(file, "file", gallery)
                        setattr(file, "parent", folder)
                        files.append(file)
            elif "performer" in str(category):
                categoryFolder = Google.get_folder_by_name(category, parent=categoryFolder)
                for performer in Google.get_folders_of_folder_by_keywords(categoryFolder):
                    # for performer in Google.get_folders_of_folder(folder):
                    p = Google_Folder()
                    setattr(p, "file", performer)
                    setattr(p, "parent", categoryFolder)
                    files.append(p)
            return files
        ##
        if performer:
            categoryFolder = Google.get_folder_by_name("performers")
            for performerFolder in Google.get_folders_of_folder_by_keywords(categoryFolder):
                if str(performer) == str(performerFolder['title']):
                    return parse_categories(cat, categoryFolder=performerFolder)
        return parse_categories(cat)

    @staticmethod
    def get_random_file():
        return random.choice(Google_File.get_files())

    def get_mimetype(self):
        if self.mimeType: return self.mimeType
        self.mimeType = self.get_file()["mimeType"]
        return self.mimeType

    def get_parent(self):
        if self.parent: return self.parent 
        self.parent = Google.get_file_parent(self.get_id())
        return self.parent

    def get_path(self):
        if self.path: return self.path
        # downloads to /tmp/downloads or whatever
        # if exists, adds 1 to end of name
        tmp = File.get_tmp()
        def counterfy():
            filename_ = str(self.get_title())+"{}"+str(self.get_ext())
            counter = 0
            while os.path.isfile(os.path.join(tmp, filename_.format(counter))):
                counter += 1
            filename_ = filename_.format(counter)
            Settings.maybe_print("filename: {}".format(filename_))
            filename_ = os.path.join(tmp, filename_.format(counter))
            return filename_
        filename = os.path.join(tmp, "{}{}".format(self.get_title(), self.get_ext()))
        if os.path.isfile(filename) and not Settings.is_prefer_local():
            filename = counterfy()
        # tmp = File.get_tmp() # i don't think this should be in file over settings
        filename = filename.strip('\'').strip('\"').strip()
        self.path = filename
        Settings.dev_print(self.path)
        return self.path

    def get_title(self):
        ## title would be set when created
        if self.title: return self.title
        title, ext = os.path.splitext(self.get_file()["title"])
        self.ext = ext
        self.title = title.replace(" ","_")
        return self.title

    # files are File references
    # file references can be GoogleId references which need to download their source
    # files exist when checked for size
    def prepare(self):
        if not self.check_size():
            self.download()
        return super()

    @staticmethod
    def select_file(category, performer=None):
        if not Settings.is_prompt(): return Google_File.get_random_file()
        # this is a list of google files to select from
        files = Google_File.get_files_by_category(category, performer=performer)
        files_ = []
        for file in files:
            if isinstance(file, str):
                files_.append(PyInquirer.Separator())
                continue
            file.category = category
            file_ = {
                "name": file.file['title'],
                "value": file,
                "short": file.file['id']
            }
            files_.append(file_)
        if len(files_) == 0:
            print("Missing Files")
            return Google_File.select_files()
        question = {
            'type': 'list',
            'name': 'file',
            'message': 'Google Files:',
            'choices': files_,
            # 'filter': lambda file: file.lower()
        }
        file = PyInquirer.prompt(question)["file"]
        if not Settings.confirm(file.get_title()): return Google_File.select_file(category)
        return file

    @staticmethod
    def select_files():
        if not Settings.is_prompt(): return [Google_File.get_random_file()]
        category = Settings.select_category()
        if not category: return File.select_file_upload_method()
        # if not Settings.confirm(category): return Google_File.select_files()
        print("Select Google Files or a Folder")
        files = []
        while True:
            file = Google_File.select_file(category)
            if not file: break
            ##
            if "performer" in str(category):
                cat = Settings.select_category([cat for cat in Settings.get_categories() if "performer" not in cat])
                performerName = file.get_title()
                file = Google_File.select_file(cat, performer=performerName)
                if not file: break
                setattr(file, "performer", performerName)
                files.append(file)
                if "galler" in str(cat) or "video" in str(cat): break
            ##
            files.append(file)
            if "galler" in str(category) or "video" in str(category): break
        if not Settings.confirm([file.file['title'] for file in files]): return Google_File.select_files()
        return files

##########################################################################################

class Google_Folder(Google_File):
    def __init__(self):
        Google_File.__init__(self)
        self.files = None

    def backup(self):
        if File.backup_text(self.get_title()): return
        Google.upload_gallery(files=self.files)

    def check_size(self):
        for file in self.get_files():
            exists = file.check_size()
            if not exists: return False
        return True

    def download(self):
        print("Downloading Folder: {}".format(self.get_title()))
        if len(self.files) == 0:
            file_list = Google.get_files_by_folder_id(self.get_id())
            self.files = []
            for file in file_list:
                file_ = Google_File()
                setattr(file_, "id", file["id"])
                setattr(file_, "file", file)
                self.files.append(file_)
        folder_size = len(self.files)
        Settings.maybe_print('Folder size: {}'.format(folder_size))
        Settings.maybe_print('Upload limit: {}'.format(Settings.get_upload_max()))
        if int(folder_size) == 0:
            print('Error: Empty Folder')
            return False
        file_list = self.files
        random.shuffle(file_list)
        file_list = file_list[:int(Settings.get_upload_max())]
        ## video preference
        videos = []
        for file in file_list:
            if str(file.get_mimetype()) in MIMETYPES_VIDEOS_LIST:
                videos.append(file)
        if len(videos) > 0: file_list = [random.choice(videos)]
        ##
        i = 1
        for file in sorted(file_list, key = lambda x: x.get_title()):
            # print_same_line("Downloading: {} ({}/{})".format(file.get_title(), i, folder_size))
            print("Downloading: {} ({}/{})".format(file.get_title(), i, folder_size))
            file.download()
            i+=1
        print()
        print("Downloaded Folder: {}".format(self.get_title()))

    def get_files(self):
        if not self.files:
            self.files = []
            files = Google.get_files_by_folder_id(self.get_id())
            for file in files:
                file_ = Google_File()
                setattr(file_, "file", file)
                self.files.append(file_)
        if Settings.get_title():
            for file in self.files:
                if str(Settings.get_title()) == str(file.get_title()):
                    self.files = [file]
                    break
        return self.files

###################################################################################

class Image(File):
    def __init__(self):
        pass

    def prepare(self):
        pass

###################################################################################

class Video(File):
    def __init__(self):
        self.screenshots = []
        self.trimmed = ""
        self.split = ""

    #seconds off front or back
    def trim(self):
        path = self.get_path()
        self.trimmed = ffmpeg.trim(path) 

    # into segments (60 sec, 5 min, 10 min)
    def split(self):
        path = self.get_path()
        self.split = ffmpeg.split(path)

    # unnecessary, handled by onlyfans
    # unless this somehow adds like more metadata
    def watermark(self):
        pass

    # cleanup & label appropriately (digital watermarking?)
    def get_metadata(self):
        pass

    # frames for preview gallery
    def get_frames(self):
        path = self.get_path()
        self.screenshots = ffmpeg.frames(path)

    def prepare(self):
        self.reduce()
        self.repair()
        self.watermark()

    def reduce(self):
        if not Settings.is_reduce(): 
            Settings.maybe_print("Skipping: Video Reduction")
            return
        path = self.get_path()
        global FIFTY_MEGABYTES
        if (int(os.stat(str(path)).st_size) < FIFTY_MEGABYTES or str(Settings.is_reduce()) == "False"):
            return
        Settings.dev_print("reduce: {}".format(self.title))
        self.path = ffmpeg.reduce(path)
    
    # unnecessary
    def repair(self):
        if not Settings.is_repair():
            Settings.dev_print("Skipping: Video Repair")
            return
        path = self.get_path()
        if Settings.is_repair():
            return
        Settings.dev_print("repair: {}".format(self.title))
        self.path = ffmpeg.repair(path)





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="OnlySnarf",
    version="2.18.0",
    author="Skeetzo",
    author_email="WebmasterSkeetzo@gmail.com",
    url = 'https://github.com/skeetzo/onlysnarf',
    keywords = ['OnlyFans', 'OnlySnarf'],
    description="OnlyFans Content Distribution Tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'selenium',
        'pydrive',
        'pathlib', 
        'chromedriver-binary',
        'geckodriver-autoinstaller',    
        'google-api-python-client',
        'httplib2',
        'python-crontab',
        'pyinquirer',
        'ffmpeg',
        'wget',
        'pysftp',
        'moviepy'
        ],
    entry_points={
        'console_scripts' : [
            'onlysnarf = OnlySnarf.bin.menu:main',
            'onlysnarfpy = OnlySnarf.src.snarf:main',
            'onlysnarf-config = OnlySnarf.bin.config:main'
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: System :: Shells',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
        "Operating System :: OS Independent",
  ]
)


#!/usr/bin/python3
# 3/25/2019: Skeetzo
# User Class

import json
import time
import os
import threading
from datetime import datetime, timedelta
from re import sub
from decimal import Decimal
##
from .colorize import colorize
from .driver import Driver
from .settings import Settings
import PyInquirer

class User:

    def __init__(self, data):
        data = json.loads(json.dumps(data))
        # print(data)
        self.name = data.get('name') or ""
        self.username = data.get('username').replace("@","") or ""
        self.id = data.get('id') or None
        self.messages_from = data.get('messages_from') or []
        self.messages_to = data.get('messages_to') or []
        self.messages = data.get('messages') or []
        self.preferences = data.get('preferences') or []
        self.last_messaged_on = data.get('last_messaged_on')
        self.sent_files = data.get('sent_files') or []
        self.subscribed_on = data.get('subscribed_on')
        self.isFavorite = data.get('isFavorite') or False
        self.statement_history = data.get('statement_history') or []
        self.started = data.get('started')
        ###### fucking json #####
        self.messages_from = ",".join(self.messages_from).split(",")
        self.messages_to = ",".join(self.messages_from).split(",")
        self.messages = ",".join(self.messages_from).split(",")
        self.preferences = ",".join(self.messages_from).split(",")
        self.sent_files = ",".join(self.messages_from).split(",")
        self.statement_history = ",".join(self.messages_from).split(",")
        #########################
        self.discount = None
        self.promotion = None
        # try:
            # Settings.dev_print("User: {} - {} - {}".format(self.name, self.username, self.id))
        # except Exception as e:
            # Settings.dev_print(e)
            # Settings.dev_print("User: {}".format(self.id))

    def discount(self, discount=None):
        if not discount: discount = Settings.get_discount()
        return Driver.discount_user(discount)

    def get_id(self):
        if self.id: return self.id
        id_ = Driver.user_get_id(self.get_username())
        self.id = id_
        return self.id

    def get_username(self):
        return self.username.replace("@", "")

    def message(self, message=None):
        if str(self.username) == "": return print("User Error: Missing Message Username")
        print("Messaging: {} - {}".format(self.username, self.id))
        successful = Driver.message(username=self.get_username(), user_id=self.id)
        if not successful: return False
        successful = User.enter_message(message)
        if not successful: return False
        print("Messaged: {}".format(self.username))

    @staticmethod
    def message_user(username="", message=None):
        user = User({"username":username})
        # setattr(user, "username", username)
        user.message(message=message)    

    @staticmethod
    def enter_message(message=None):
        try:
            print("Entering Message: {} - ${}".format(message.text, message.get_price() or 0))
            def enter_text(text):
                success = Driver.message_text(text)
                if not success: return False
                return True
            def enter_price(price):
                if price == "": return False
                if price != None and Decimal(sub(r'[^\d.]', '', price)) < Settings.get_price_minimum():
                    print("Warning: Price Too Low, Free Image")
                    print("Price Minimum: ${}".format(Settings.get_price_minimum()))
                else:
                    success = Driver.message_price(price)
                    if not success: return False
                Settings.debug_delay_check()
                return True
            def enter_files(files):
                # file_name = os.path.basename(path)
                # if str(file_name) in self.sent_files:
                    # print("Error: File Already Sent: {} -> {}".format(file_name, self.username))
                    # return False
                success = Driver.message_files(files)
                if not success: return False
                # if not Settings.is_debug():
                    # self.sent_files.append(str(file_name))
                Settings.debug_delay_check()
                return True
            def confirm():
                success = Driver.message_confirm()
                if not success: return False
                return True
            if not enter_text(message.get_text()): return False # not allowed to fail
            enter_price(message.get_price()) # allowed to fail
            enter_files(message.get_files()) # allowed to fail
            if not confirm(): return False # not allowed to fail
            print("Message Entered")
            return True
        except Exception as e:
            Settings.dev_print(e)
            return False

    def equals(self, user):
        # print(str(user.username)+" == "+str(self.username))
        if user.username == self.username:
            return True
        return False

    def toJSON(self):
        return json.dumps({
            "name":str(self.name),
            "username":str(self.username),
            "id":str(self.id),
            "messages_from":str(self.messages_from),
            "messages_to":str(self.messages_to),
            "messages":str(self.messages),
            "preferences":str(self.preferences),
            "last_messaged_on":str(self.last_messaged_on),
            "sent_files":str(self.sent_files),
            "subscribed_on":str(self.subscribed_on),
            "isFavorite":str(self.isFavorite)
        })

    # def get_username(self):
    #     return self["username"]

    # greet user if new
    def greet(self):
        if self.last_messaged_on == None:
            return print("Error: User Not New")
        print("Sending User Greeting: {}".format(self.username))
        # self.send_message(message=Settings.DEFAULT_GREETING)
        User.enter_message(message=Settings.get_default_greeting())

    # send refresher message to user
    def refresh(self):
        if self.last_messaged_on == None:
            print("Warning: Never Greeted, Greeting Instead")
            return self.greet()
        elif (timedelta(self.last_messaged_on)-timedelta(datetime())).days < 30:
            return print("Error: Refresher Date Too Early - {}".format((timedelta(self.last_messaged_on)-timedelta(datetime())).days))
        print("Sending User Refresher: {}".format(self.username))
        # self.send_message(message=Settings.user_DEFAULT_REFRESHER)
        User.enter_message(message=Settings.get_default_refresher())

    # saves chat log to user
    def readChat(self, Driver):
        print("Reading Chat: {} - {}".format(self.username, self.id))
        messages = Driver.read_user_messages(self.id)
        self.messages = messages[0]
        # self.messages_and_timestamps = messages[1]
        self.messages_to = messages[2]
        self.messages_from = messages[3]
        Settings.maybe_print("Chat Read: {} - {}".format(self.username, self.id))

    # saves statement / payment history
    def statementHistory(self, history):
        print("Reading Statement History: {} - {}".format(self.username, self.id))
        Driver.read_statements(user=self.id)

    # sets as favorite
    def favor(self):
        print("Favoring: {}".format(self.username))
        self.isFavorite = True

    # unsets as favorite
    def unfavor(self):
        print("Unfavoring: {}".format(self.username))
        self.isFavorite = False

    @staticmethod
    def get_all_users():
        return User.get_active_users()

    # gets users from local or refreshes from onlyfans.com
    @staticmethod
    def get_active_users():
        if Settings.is_prefer_local(): return User.read_users_local()
        active_users = []
        users = Driver.users_get()
        for user in users:
            try:
                user = User(user)
                user = User.skipUserCheck(user)
                if user is None: continue
                active_users.append(user)
            except Exception as e:
                Settings.dev_print(e)
        Settings.maybe_print("pruning memberlist")
        Settings.maybe_print("users: {}".format(len(active_users)))
        User.write_users_local(users=active_users)
        Settings.set_prefer_local(True)
        return active_users

    @staticmethod
    # return following Users
    def get_following():
        if Settings.is_prefer_local_following(): return User.read_following_local()
        active_users = []
        users = Driver.following_get()
        for user in users:
            try:
                user = User(user)
                user = User.skipUserCheck(user)
                if user is None: continue
                active_users.append(user)
            except Exception as e:
                Settings.dev_print(e)
        Settings.maybe_print("following: {}".format(len(active_users)))
        User.write_following_local(users=active_users)
        Settings.set_prefer_local_following(True)
        return active_users

    @staticmethod
    def get_user_by_username(username):
        if not username or username == None:
            print("Error: Missing Username")
            return None
        users = User.read_users_local()
        for user in users:
            if str(user.username) == "@u"+str(username) or str(user.username) == "@"+str(username) or str(user.username) == str(username):
                Settings.maybe_print("Found User: Local")
                return user
        users = User.get_all_users()
        for user in users:
            if str(user.username) == "@u"+str(username) or str(user.username) == "@"+str(username) or str(user.username) == str(username):
                Settings.maybe_print("Found User: Members")
                return user
        print("Error: Missing User by Username - {}".format(username))
        return None

    @staticmethod
    def get_favorite_users():
        Settings.maybe_print("Getting Favorite Users")
        users = User.get_all_users()
        favUsers = []
        favorites = ",".join(str(Settings.get_users_favorite()))
        for user in users:
            if user in favorites:
                Settings.maybe_print("Fav User: {}".format(user.username))
                user = User.skipUserCheck(user)
                if user is None: continue
                favUsers.append(user)
        return favUsers

    # returns users that have no messages sent to them
    @staticmethod
    def get_new_users():
        Settings.maybe_print("Getting New Users")
        users = User.get_all_users()
        newUsers = []
        date_ = datetime.today() - timedelta(days=10)
        for user in users:
            if not user.started: continue
            started = datetime.strptime(str(user.started),"%b %d, %Y")
            # Settings.maybe_print("date: "+str(date_)+" - "+str(started))
            if started < date_: continue
            Settings.maybe_print("New User: {}".format(user.username))
            user = User.skipUserCheck(user)
            if user is None: continue
            newUsers.append(user)
        return newUsers

    @staticmethod
    def get_never_messaged_users():
        Settings.maybe_print("Getting New Users")
        update_chat_logs()
        users = User.get_all_users()
        newUsers = []
        for user in users:
            if len(user.messages_to) == 0:
                Settings.maybe_print("Never Messaged User: {}".format(user.username))
                user = User.skipUserCheck(user)
                if user is None: continue
                newUsers.append(user)
        return newUsers

    @staticmethod
    def get_recent_users():
        Settings.maybe_print("Getting Recent Users")
        users = User.get_all_users()
        i = 0
        users_ = []
        for user in users:
            Settings.maybe_print("Recent User: {}".format(user.username))
            user = User.skipUserCheck(user)
            if user is None: continue
            users_.append(user)
            i += 1
            if i == int(Settings.get_recent_user_count()):
                return users_
        return users_

    @staticmethod
    def select_user():
        user = Settings.get_user() or None
        if user: return user
        # if not Settings.prompt("user"): return None
        choices = Settings.get_message_choices()
        choices.append("enter username")
        choices.append("select username")
        choices = [str(choice).title() for choice in choices]
        question = {
            'type': 'list',
            'name': 'user',
            'message': 'User:',
            'choices': choices,
            'filter': lambda val: str(val).lower()
        }
        answers = PyInquirer.prompt(question)
        user = answers["user"]
        if str(user) == "enter username":
            username = input("Username: ")
            return User.get_user_by_username(username)
        elif str(user) == "select username":
            return User.select_username()
        elif str(user) == "favorites":
            return User.get_favorite_users()
        if not Settings.confirm(user): return User.select_user()
        return user

    @staticmethod
    def select_users():
        # if not Settings.prompt("users"): return []
        users = []
        while True:
            user = User.select_user()
            if not user: break
            if str(user).lower() == "all" or str(user).lower() == "recent": return [user]
            users.append(user)
            if not Settings.prompt("another user"): break
        if not Settings.confirm([user.username for user in users]): return User.select_users()
        return users

    @staticmethod
    def select_username():
        # returns the list of usernames to select
        # if not Settings.prompt("select username"): return None
        users = User.get_all_users()
        users_ = []
        for user in users:
            user_ = {
                "name":user.username.replace("@",""),
                "value":user,
                "short":user.id
            }
            users_.append(user_)
        question = {
            'type': 'list',
            'name': 'user',
            'message': 'Username:',
            'choices': users_
        }
        user = PyInquirer.prompt(question)['user']
        if not Settings.confirm(user.username): return User.select_username()
        return user

    # gets a list of all subscribed user_ids from local txt
    @staticmethod
    def read_users_local():
        Settings.maybe_print("getting local users")
        users = []
        users_ = []
        try:
            with open(str(Settings.get_users_path())) as json_file:  
                users = json.load(json_file)['users']
            Settings.maybe_print("loaded local users")
            for user in users:
                try:
                    users_.append(User(json.loads(user)))
                except Exception as e:
                    Settings.dev_print(e)
            return users_
        except Exception as e:
            Settings.dev_print(e)
        return users_

    @staticmethod
    def skipUserCheck(user):
        if str(user.id).lower() in Settings.get_skipped_users() or str(user.username).lower() in Settings.get_skipped_users():
            Settings.maybe_print("skipping: {}".format(user.username))
            return None
        return user

    # writes user list to local txt
    @staticmethod
    def write_users_local(users=None):
        if users is None:
            users = User.get_all_users()
        if len(users) == 0:
            Settings.maybe_print("Skipping: Local Users Save - No Users")
            return
        print("Saving Users Locally")
        Settings.maybe_print("local users path: "+str(Settings.get_users_path()))
        data = {}
        data['users'] = []
        for user in users:
            data['users'].append(user.toJSON())
        try:
            with open(str(Settings.get_users_path()), 'w') as outfile:  
                json.dump(data, outfile, indent=4, sort_keys=True)
        except FileNotFoundError:
            print("Error: Missing Local Users")
        except OSError:
            print("Error: Missing Local Path")

    @staticmethod
    def read_following_local():
        Settings.maybe_print("Getting Local Following")
        users = []
        users_ = []
        try:
            with open(str(Settings.get_users_path().replace("users.json", "following.json"))) as json_file:  
                users = json.load(json_file)['users']
            Settings.maybe_print("Loaded Local Following")
            for user in users:
                try:
                    users_.append(User(json.loads(user)))
                except Exception as e:
                    Settings.dev_print(e)
            return users_
        except Exception as e:
            Settings.dev_print(e)
        return users_

    @staticmethod
    def write_following_local(users=None):
        if users is None:
            users = User.get_following()
        if len(users) == 0:
            Settings.maybe_print("Skipping: Local Following Save - No Following")
            return
        print("Saving Following Users Locally")
        Settings.maybe_print("local users path: "+str(Settings.get_users_path().replace("users.json", "following.json")))
        data = {}
        data['users'] = []
        for user in users:
            data['users'].append(user.toJSON())
        try:
            with open(str(Settings.get_users_path().replace("users.json", "following.json")), 'w') as outfile:  
                json.dump(data, outfile, indent=4, sort_keys=True)
        except FileNotFoundError:
            print("Error: Missing Local Following")
        except OSError:
            print("Error: Missing Local Path")

#######################################################################################

def delayForThirty():
    Settings.maybe_print("30...")
    time.sleep(10)
    Settings.maybe_print("20...")
    time.sleep(10)
    Settings.maybe_print("10...")
    time.sleep(7)
    Settings.maybe_print("3...")
    time.sleep(1)
    Settings.maybe_print("2...")
    time.sleep(1)
    Settings.maybe_print("1...")
    time.sleep(1)
